package com.ibm.xml.internal;

import com.ibm.xml.framework.EntityDecl;
import com.ibm.xml.framework.EntityPool;
import com.ibm.xml.framework.NotationDecl;
import com.ibm.xml.framework.ParserState;
import com.ibm.xml.framework.StringPool;
import com.ibm.xml.framework.XMLErrorHandler;

public final class DefaultEntityPool implements EntityPool {
   private static final boolean DEBUG_530 = false;
   static final int CHUNK_SHIFT = 5;
   static final int CHUNK_SIZE = 32;
   static final int CHUNK_MASK = 31;
   static final int INITIAL_CHUNK_COUNT = 32;
   private ParserState fParserState;
   private StringPool fStringPool;
   private int fEntityCount;
   private int[][] fEntityName = new int[32][];
   private int[][] fEntityValue = new int[32][];
   private int[][] fPublicId = new int[32][];
   private int[][] fSystemId = new int[32][];
   private int[][] fNotationName = new int[32][];
   private int fNotationListHead = -1;
   private EntityDecl fInternalEntityDecl;

   public DefaultEntityPool(ParserState var1, boolean var2) {
      this.fParserState = var1;
      this.fStringPool = var1.cacheStringPool();
      if (var2) {
         this.fInternalEntityDecl = new EntityDecl();
         this.createInternalEntity("lt", "&#60;");
         this.createInternalEntity("gt", ">");
         this.createInternalEntity("amp", "&#38;");
         this.createInternalEntity("apos", "'");
         this.createInternalEntity("quot", "\"");
      }

   }

   public void reset(ParserState var1) {
      this.fParserState = var1;
      this.fStringPool = var1.cacheStringPool();
      this.fEntityCount = 0;
      this.fNotationListHead = -1;
      if (this.fInternalEntityDecl != null) {
         this.createInternalEntity("lt", "&#60;");
         this.createInternalEntity("gt", ">");
         this.createInternalEntity("amp", "&#38;");
         this.createInternalEntity("apos", "'");
         this.createInternalEntity("quot", "\"");
      }

   }

   public EntityPool resetOrCopy(ParserState var1) {
      return new DefaultEntityPool(var1, this.fInternalEntityDecl != null);
   }

   private void createInternalEntity(String var1, String var2) {
      this.fInternalEntityDecl.entityName = this.fStringPool.addSymbol(var1);
      this.fInternalEntityDecl.entityValue = this.fStringPool.addString(var2);
      this.fInternalEntityDecl.publicId = -1;
      this.fInternalEntityDecl.systemId = -1;
      this.fInternalEntityDecl.notationName = -1;
      this.addEntityDecl(this.fInternalEntityDecl);
   }

   private boolean ensureCapacity(int var1) {
      try {
         if (this.fEntityName[var1][0] != 0) {
            return false;
         }

         return true;
      } catch (ArrayIndexOutOfBoundsException var3) {
         int[][] var2 = new int[var1 * 2][];
         System.arraycopy(this.fEntityName, 0, var2, 0, var1);
         this.fEntityName = var2;
         var2 = new int[var1 * 2][];
         System.arraycopy(this.fEntityValue, 0, var2, 0, var1);
         this.fEntityValue = var2;
         var2 = new int[var1 * 2][];
         System.arraycopy(this.fPublicId, 0, var2, 0, var1);
         this.fPublicId = var2;
         var2 = new int[var1 * 2][];
         System.arraycopy(this.fSystemId, 0, var2, 0, var1);
         this.fSystemId = var2;
         var2 = new int[var1 * 2][];
         System.arraycopy(this.fNotationName, 0, var2, 0, var1);
         this.fNotationName = var2;
      } catch (NullPointerException var4) {
      }

      this.fEntityName[var1] = new int[32];
      this.fEntityValue[var1] = new int[32];
      this.fPublicId[var1] = new int[32];
      this.fSystemId[var1] = new int[32];
      this.fNotationName[var1] = new int[32];
      return true;
   }

   public int addEntityDecl(EntityDecl var1) {
      int var2 = this.fEntityCount >> 5;
      int var3 = this.fEntityCount & 31;
      this.ensureCapacity(var2);
      this.fEntityName[var2][var3] = var1.entityName;
      this.fEntityValue[var2][var3] = var1.entityValue;
      this.fPublicId[var2][var3] = var1.publicId;
      this.fSystemId[var2][var3] = var1.systemId;
      this.fNotationName[var2][var3] = var1.notationName;
      int var4 = this.fEntityCount++;
      return var4;
   }

   public int addNotationDecl(NotationDecl var1) {
      int var3;
      int var4;
      for(int var2 = this.fNotationListHead; var2 != -1; var2 = this.fEntityValue[var3][var4]) {
         var3 = var2 >> 5;
         var4 = var2 & 31;
         if (this.fNotationName[var3][var4] == var1.notationName) {
            return -1;
         }
      }

      var3 = this.fEntityCount >> 5;
      var4 = this.fEntityCount & 31;
      this.ensureCapacity(var3);
      this.fEntityName[var3][var4] = -1;
      this.fEntityValue[var3][var4] = this.fNotationListHead;
      this.fPublicId[var3][var4] = var1.publicId;
      this.fSystemId[var3][var4] = var1.systemId;
      this.fNotationName[var3][var4] = var1.notationName;
      this.fNotationListHead = this.fEntityCount++;
      return this.fNotationListHead;
   }

   public int lookupEntity(int var1) {
      int var2 = 0;
      int var3 = 0;

      for(int var4 = 0; var4 < this.fEntityCount; ++var4) {
         if (this.fEntityName[var2][var3] == var1) {
            return var4;
         }

         ++var3;
         if (var3 == 32) {
            ++var2;
            var3 = 0;
         }
      }

      return -1;
   }

   public boolean isExternal(int var1) {
      int var2 = var1 >> 5;
      int var3 = var1 & 31;
      return this.fEntityValue[var2][var3] == -1;
   }

   public boolean isUnparsed(int var1) {
      int var2 = var1 >> 5;
      int var3 = var1 & 31;
      return this.fNotationName[var2][var3] != -1;
   }

   public int getEntityName(int var1) {
      int var2 = var1 >> 5;
      int var3 = var1 & 31;
      return this.fEntityName[var2][var3];
   }

   public int getEntityValue(int var1) {
      int var2 = var1 >> 5;
      int var3 = var1 & 31;
      return this.fEntityValue[var2][var3];
   }

   public int getPublicId(int var1) {
      int var2 = var1 >> 5;
      int var3 = var1 & 31;
      return this.fPublicId[var2][var3];
   }

   public int getSystemId(int var1) {
      int var2 = var1 >> 5;
      int var3 = var1 & 31;
      return this.fSystemId[var2][var3];
   }

   public int lookupNotation(int var1) {
      int var3;
      int var4;
      for(int var2 = this.fNotationListHead; var2 != -1; var2 = this.fEntityValue[var3][var4]) {
         var3 = var2 >> 5;
         var4 = var2 & 31;
         if (this.fNotationName[var3][var4] == var1) {
            return var2;
         }
      }

      return -1;
   }

   public int getNotationName(int var1) {
      int var2 = var1 >> 5;
      int var3 = var1 & 31;
      return this.fNotationName[var2][var3];
   }

   public void checkUnparsedEntities() throws Exception {
      XMLErrorHandler var1 = this.fParserState.getErrorHandler();

      for(int var2 = 0; var2 < this.fEntityCount; ++var2) {
         int var3 = var2 >> 5;
         int var4 = var2 & 31;
         if (this.fEntityName[var3][var4] != -1) {
            int var5 = this.fNotationName[var3][var4];
            if (var5 != -1 && this.lookupNotation(var5) == -1) {
               var1.error1(68, var5);
            }
         }
      }

   }
}
