package com.ibm.xml.internal;

import com.ibm.xml.framework.ParserState;
import com.ibm.xml.framework.StringPool;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class XCatalog extends DefaultCatalog {
   public static final String XCATALOG_DTD_PUBLICID = "-//DTD XCatalog//EN";
   static final String DTD = "xcatalog.dtd";
   static final String XCATALOG = "XCatalog";
   static final String MAP = "Map";
   static final String PUBLICID = "PublicID";
   static final String HREF = "HRef";
   static final String DELEGATE = "Delegate";
   static final String EXTEND = "Extend";
   static final String BASE = "Base";
   static final String REMAP = "Remap";
   static final String SYSTEMID = "SystemID";
   private static final boolean DEBUG = false;
   private Hashtable delegate = new Hashtable();
   private Vector delegateOrder = new Vector();
   private ParserState state;

   public XCatalog(ParserState var1) {
      this.state = var1;
   }

   public void loadCatalog(InputSource var1) throws SAXException, IOException {
      new Parser(this, var1);
   }

   public InputSource resolveEntity(String var1, String var2) throws SAXException, IOException {
      if (var1 != null) {
         String var3 = ((DefaultCatalog)this).getPublicMapping(var1);
         if (var3 != null) {
            InputSource var10 = this.resolveEntity((String)null, var3);
            if (var10 == null) {
               var10 = new InputSource(var3);
            }

            var10.setPublicId(var1);
            return var10;
         }

         Enumeration var4 = this.getDelegateCatalogKeys();

         while(var4.hasMoreElements()) {
            String var5 = (String)var4.nextElement();
            if (var1.startsWith(var5)) {
               XCatalog var6 = this.getDelegateCatalog(var5);
               InputSource var7 = var6.resolveEntity(var1, var2);
               if (var7 != null) {
                  return var7;
               }
            }
         }
      }

      String var8 = ((DefaultCatalog)this).getSystemMapping(var2);
      if (var8 != null) {
         InputSource var9 = new InputSource(var8);
         var9.setPublicId(var1);
         return var9;
      } else {
         return null;
      }
   }

   public void addDelegateCatalog(String var1, XCatalog var2) {
      Hashtable var3 = this.delegate;
      synchronized(var3){}

      try {
         if (!this.delegate.containsKey(var1)) {
            int var5 = this.delegateOrder.size();
            boolean var6 = false;

            for(int var7 = 0; var7 < var5; ++var7) {
               String var8 = (String)this.delegateOrder.elementAt(var7);
               if (var1.startsWith(var8) || var1.compareTo(var8) < 0) {
                  this.delegateOrder.insertElementAt(var1, var7);
                  var6 = true;
                  break;
               }
            }

            if (!var6) {
               this.delegateOrder.addElement(var1);
            }
         }

         this.delegate.put(var1, var2);
      } catch (Throwable var10) {
         throw var10;
      }

   }

   public void removeDelegateCatalog(String var1) {
      Hashtable var2 = this.delegate;
      synchronized(var2){}

      try {
         this.delegate.remove(var1);
         this.delegateOrder.removeElement(var1);
      } catch (Throwable var4) {
         throw var4;
      }

   }

   public Enumeration getDelegateCatalogKeys() {
      return this.delegateOrder.elements();
   }

   public XCatalog getDelegateCatalog(String var1) {
      return (XCatalog)this.delegate.get(var1);
   }

   protected ParserState getParserState() {
      return this.state;
   }

   String expandSystemId(ParserState var1, String var2) {
      StringPool var3 = var1.getStringPool();
      int var4 = var3.addString(var2);
      int var5 = var1.getEntityHandler().expandSystemId(var4);
      if (var5 != var4) {
         var2 = var3.orphanString(var5);
      }

      var3.releaseString(var4);
      return var2;
   }

   boolean isURL(String var1) {
      try {
         new URL(var1);
         return true;
      } catch (MalformedURLException var2) {
         return false;
      }
   }

   // $FF: synthetic method
   static boolean access$0() {
      return false;
   }
}
